@@ -1,6 +1,7 @@
 import functools
 import json
 from collections import Counter
+from contextlib import contextmanager
 from typing import List, Union
 
 from metrics_layer.core.exceptions import (
@@ -177,7 +178,8 @@ class Project:
 
         return copied_views
 
-    def validate_with_replaced_objects(self, replaced_objects: list):
+    @contextmanager
+    def replace_objects(self, replaced_objects: list):
         replaced_views, replaced_models, replaced_dashboards = [], [], []
         for dict_obj in replaced_objects:
             if isinstance(dict_obj, dict):
@@ -195,28 +197,36 @@ class Project:
         replaced_model_names = set([m["name"] for m in replaced_models])
         unchanged_models = [m for m in self._models if m["name"] not in replaced_model_names]
         current_models = json.loads(json.dumps(self._models))
-        self._models = unchanged_models + replaced_models
 
         # Replace view files
         replaced_view_names = set([v["name"] for v in replaced_views])
         unchanged_views = [v for v in self._views if v["name"] not in replaced_view_names]
         current_views = json.loads(json.dumps(self._views))
-        self._views = unchanged_views + replaced_views
 
         # Replace dashboard files
         replaced_dashboard_names = set([d["name"] for d in replaced_dashboards])
         unchanged_dashboards = [d for d in self._dashboards if d["name"] not in replaced_dashboard_names]
         current_dashboards = json.loads(json.dumps(self._dashboards))
-        self._dashboards = unchanged_dashboards + replaced_dashboards
-        self.refresh_cache()
 
-        errors = self.validate()
+        try:
+            self._models = unchanged_models + replaced_models
+            self._views = unchanged_views + replaced_views
+            self._dashboards = unchanged_dashboards + replaced_dashboards
+            self.refresh_cache()
+            yield
+        finally:
+            self._dashboards = current_dashboards
+            self._views = current_views
+            self._models = current_models
+            self.refresh_cache()
 
-        self._views = current_views
-        self._models = current_models
-        self._dashboards = current_dashboards
-        self.refresh_cache()
-        return errors
+    def validate_with_replaced_objects(self, replaced_objects: list):
+        with self.replace_objects(replaced_objects):
+            return self.validate()
+
+    def _error(self, error: str, extra: dict = {}):
+        # For project level errors we cannot attribute a line or column
+        return {**extra, "message": error, "line": None, "column": None}
 
     def validate(self):
         all_errors = []
@@ -225,14 +235,14 @@ class Project:
                 all_errors.extend(model.collect_errors())
             except QueryError as e:
                 # If we have an error building the model, we cannot continue
-                return [str(e)]
+                return [self._error(str(e))]
 
         try:
             all_errors.extend(self.join_graph.collect_errors())
         except QueryError as e:
             # If we have an error building the graph, we cannot continue
             # and no other errors will be relevant until this is fixed
-            return [str(e)]
+            return [self._error(str(e))]
 
         for join_graph in self.join_graph.list_join_graphs():
             try:
@@ -240,7 +250,7 @@ class Project:
             except QueryError as e:
                 error_text = str(e).replace(" name ", " tag ").split("\n")[0]
                 error_text += '. Only one field can have the tag "customer" per joinable graph.'
-                all_errors.append(error_text)
+                all_errors.append(self._error(error_text))
             except Exception:
                 pass
 
@@ -249,23 +259,33 @@ class Project:
                 for user_attribute_name in self._required_access_filter_user_attributes:
                     if not view.access_filters:
                         all_errors.append(
-                            f"View {view.name} does not have any access filters, but an access filter with"
-                            f" user attribute {user_attribute_name} is required."
+                            self._error(
+                                (
+                                    f"View {view.name} does not have any access filters, but an access filter"
+                                    f" with user attribute {user_attribute_name} is required."
+                                ),
+                                {"view_name": view.name},
+                            )
                         )
                     elif all(af["user_attribute"] != user_attribute_name for af in view.access_filters):
                         all_errors.append(
-                            f"View {view.name} does not have an access filter with the required user"
-                            f" attribute {user_attribute_name}"
+                            self._error(
+                                (
+                                    f"View {view.name} does not have an access filter with the required user"
+                                    f" attribute {user_attribute_name}"
+                                ),
+                                {"view_name": view.name},
+                            )
                         )
 
             try:
                 view.sql_table_name
             except QueryError as e:
-                all_errors.append(str(e) + f" in the view {view.name}")
+                all_errors.append(self._error(str(e) + f" in the view {view.name}", {"view_name": view.name}))
             try:
                 referenced_fields = view.referenced_fields()
             except (AccessDeniedOrDoesNotExistException, QueryError) as e:
-                all_errors.append(str(e) + f" in the view {view.name}")
+                all_errors.append(self._error(str(e) + f" in the view {view.name}", {"view_name": view.name}))
 
             view_errors = view.collect_errors()
 
@@ -280,8 +300,13 @@ class Project:
                         field_reference = field[-1]
                         prepend = ""
                     all_errors.append(
-                        f"{prepend}Could not locate reference {field_reference} in field {field_name} in view"
-                        f" {view.name}"
+                        self._error(
+                            (
+                                f"{prepend}Could not locate reference {field_reference} in field"
+                                f" {field_name} in view {view.name}"
+                            ),
+                            {"view_name": view.name, "field_name": field_name},
+                        )
                     )
             all_errors.extend(view_errors)
 
@@ -291,7 +316,13 @@ class Project:
 
         all_errors.extend(self._validate_dashboard_names())
 
-        return list(sorted(set(all_errors), key=lambda x: all_errors.index(x)))
+        cleaned_errors, _seen = [], set([])
+        for e in all_errors:
+            if isinstance(e, dict) and e["message"] not in _seen:
+                cleaned_errors.append(e)
+                _seen.add(e["message"])
+
+        return cleaned_errors
 
     def _validate_dashboard_names(self):
         # We need to make sure the unique identifiers for the dashboards are actually unique
@@ -301,7 +332,7 @@ class Project:
         for name, frequency in name_frequency:
             if frequency > 1:
                 msg = f"Dashboard name {name} appears {frequency} times, make sure dashboard names are unique"
-                errors.append(msg)
+                errors.append(self._error(msg))
             else:
                 break
         return errors
@@ -342,7 +373,7 @@ class Project:
             )
 
     def access_grants(self):
-        return [AccessGrant(g) for m in self.models() for g in m.access_grants]
+        return [AccessGrant(g, model=m) for m in self.models() for g in m.access_grants]
 
     def get_access_grant(self, grant_name: str):
         try:
@@ -399,6 +430,9 @@ class Project:
                 object_type="view",
             )
 
+    def get_joinable_views(self, view_name: str) -> List[str]:
+        return self.join_graph.get_joinable_view_names(view_name)
+
     def sets(self, view_name: Union[str, None] = None):
         if view_name:
             try:
